package com.me.asteroids.components.model;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Polyline;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;

public class LineModel implements Model {

    private Vector2 tmp = new Vector2();

    private Color color;

    private Polyline model;
    private Rectangle aabb;

    private boolean dirty = true;

    public LineModel(Color color) {
        this.color = color;
        this.model = new Polyline();
        this.aabb = new Rectangle();
    }

    @Override
    public Color getColor() {
        return color;
    }

    @Override
    public void setVertices(float[] vertices) {
        model.setVertices(vertices);
    }

    @Override
    public float[] getVertices() {
        return model.getTransformedVertices();
    }

    @Override
    public void setPosition(Vector2 position) {
        model.setPosition(position.x, position.y);
        dirty = true;
    }

    @Override
    public Vector2 getPosition() {
        return tmp.set(model.getX(), model.getY());
    }

    @Override
    public void setRotation(float degrees) {
        model.setRotation(degrees);
        dirty = true;
    }

    @Override
    public float getRotation() {
        return model.getRotation();
    }

    @Override
    public void setScale(float scale) {
        model.setScale(scale, scale);
        dirty = true;
    }

    @Override
    public float getScale() {
        return model.getScaleX();
    }

    @Override
    public Rectangle getBoundingBox() {
        if (dirty) {
            updateBoundingBox();
            dirty = false;
        }
        return aabb;
    }

    @Override
    public void render(ShapeRenderer renderer) {
        renderer.polyline(getVertices());
    }

    private void updateBoundingBox() {
        float[] vertices = getVertices();
        float minX = vertices[0];
        float minY = vertices[1];
        float maxX = vertices[0];
        float maxY = vertices[1];

        for (int i = 2, n = vertices.length; i < n; i += 2) {
            float x = vertices[i];
            float y = vertices[i+1];
            minX = x < minX ? x : minX;
            maxX = x > maxX ? x : maxX;
            minY = y < minY ? y : minY;
            maxY = y > maxY ? y : maxY;
        }

        aabb.x = minX;
        aabb.y = minY;
        aabb.width = maxX - minX;
        aabb.height = maxY - minY;
    }


}
