package com.me.asteroids.components.model;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Polygon;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;

public class PolygonModel implements Model {

    private Vector2 tmp = new Vector2();

    private Color color;

    private Polygon model;
    private Rectangle aabb;

    private boolean dirty = true;

    public PolygonModel(Color color) {
        this.color = color;
        this.model = new Polygon();
    }

    @Override
    public Color getColor() {
        return color;
    }

    @Override
    public void setVertices(float[] vertices) {
        model.setVertices(vertices);
    }

    @Override
    public float[] getVertices() {
        return model.getTransformedVertices();
    }

    @Override
    public void setPosition(Vector2 position) {
        model.setPosition(position.x, position.y);
        dirty = true;
    }

    @Override
    public Vector2 getPosition() {
        return tmp.set(model.getX(), model.getY());
    }

    @Override
    public void setRotation(float degrees) {
        model.setRotation(degrees);
        dirty = true;
    }

    @Override
    public float getRotation() {
        return model.getRotation();
    }

    @Override
    public void setScale(float scale) {
        model.setScale(scale, scale);
        dirty = true;
    }

    @Override
    public float getScale() {
        return model.getScaleX();
    }

    @Override
    public Rectangle getBoundingBox() {
        if (dirty) {
            aabb = model.getBoundingRectangle();
            dirty = false;
        }
        return aabb;
    }

    @Override
    public void render(ShapeRenderer renderer) {
        renderer.polygon(getVertices());
    }

    public boolean contains(Vector2 point) {
        return model.contains(point);
    }

    public boolean contains(float x, float y) {
        return model.contains(x, y);
    }

    public boolean contains(float[] vertices) {
        for (int i = 0, n = vertices.length; i < n; i += 2) {
            if (model.contains(vertices[i], vertices[i + 1])) {
                return true;
            }
        }
        return false;
    }

}
